#!/usr/bin/env python

# This is the convusb program, inputs hex character stream (like what's
# dumped by Kai Wang's krepdump.ko kernel module, which has a 4 place hex
# address field, then a single space, then groups of 2 digit bytes) and
# puts out parsed HID code, followed by the hex bytes that drove that line.
# Flag=  -f argument, which changes the input from stdin to whatever comes after
# the -f option 
#
# Thanks go to both Kai Wang and Markus Brueffer, two denizens of the FreeBSD-USB
# mailing list who went WAY out of their way to be of help to me in learning
# what I think is the worst spec I have ever bumped up against.
# Chuck Robey (chuckr@chuckr.org).
# Get the latest version at http://people.freebsd.org/~chuckr/code/python/convusb.py-1.0

import copy

class tables(object):
    tablename = '';

    def getUsages(self, index):
        return (self.tablename)[index]

    def getUsage(self, index1):
        self.tablename =  usages[index1][1]
        return usages[index1]

    def getMainItems(self, index):
        return self.mainItems[index]

    def getMainCodes(self, index1):
        return self.mainCodes[index1]
    
    # All that follows are masses of tables, mostly dictionaries.

    field = {
        'Addr':"""Since data rows are 16 bytes (32 hex ASCII chars) in length, these are the 4 ASCII-hex byte row addresses (0000, 0010, etc).  More than 4 bytes of address are stripped off.  They are stripped off before assembling items, but saved for this display.  This is only updated row by row, NOT item by item, it's the row address.  If you have a binary file to input, run the Unix 'od -t xC' utility on it, then send that output to this program.""",
        'InputLineBegin':"""Beginning empty, every time a set of hex chars are assembled, they are placed here after they are used.  This is emptied on each new line, so the illusion of working with line by line input is maintained as much as possible, because, after all, this is designed to work with input lines having a 4 char ASCII address, a space, then 16 triplets of 2 ASCII hex char byte values and 1 space, representing 16 full bytes of info per line.  The idea is to provide a simple way to graphically partition each line to the part already  processed, the active command part, the data part, and the part yet to be used, all in the same gui display field.  If your data is binary, use the Unix "od" utility with "-t Xc" as arguments.""",
        'InputLineCommand':"""Part 2 (of 4 parts) making up an entire line (and shown in blue background), this is the part of an item carrying the bTag, bType, and bSize commands, used in graphically displaying each entire line, with the command and data parts emphasized, and seemingly moving as your scroll the input forward.  Not too useful for a NonGUI interface.""",
        'InputLineData':"""This is part 3 of a 4 part scheme to display the full current input line, with a moving emphasized window of the current command (shown in blue background) and data pieces (shown with a yellow background).  This is the current data, and sized by the bSize bits of the first byte (the bottom 2 bits, coded as 0->0 bytes, 1->1 byte, 2->2 byte, and 3->4 byte (that's no typo).   The Begin And Ending pieces grow and wane, accordingly, as the line is processed.  The gui has to worry about display.""",
        'InputLineEnding':"""This is the final part of the current input line, sent as Begin, current command, current data, and Ending, so the current line can be shown with a moving colored indication of where the part being actively scoped is.""",
        'bTag':"""This leading nibble of each item's first byte indicates (once you figure the command Class) what the exact type of item this is.  To decode this, you first figure the command class (Main, Global. or Local) then use bTag values to get the actual command.  Look to "Name" for decoding of this.""",
        'bType':"""The top 2 bit piece of the lower nibble of the upper byte of each item, decoded, gives you the type of command, decoded as "Class", either Main, Global, or Local.""",
        'bSize':"""This is the bottom 2 bits of the top byte, and via a table, indicated the size (in bytes) of the following data part, 0,1,2,3 means 0,1,2,4 (no typo) bytes of data.""",
        'Class':"""This is the decoding of the bType, indicating what type of item this is, one of Main, Global, or Local.  Without this decoding, one cannot use the bTag field to determine the Item Name, because not all bTag values are unique among the Class values.""",
        'Name':"""This is the name of the item, derived by decoding the Class, and then the bTag value.""",
        'DataValue':"""These are the ASCII bytes representing, in hex, the bytes of the item's data, length signaled via the bSize field, may be 0 length.  Note that this data is in network order, and needs to be flipped before being read.""",
        'DataLength':"""This is the decoding of the bSize field, 0,1,2,3 means 0,1,2, and 4 (that's not a typo).""",
        'DecimalData':"""Since the Data field (which is in ASCII) is limited to hex values, and much of the time this only makes sense in decimal, this is the decimal decoding of the Data field.""",
        'UnitExponent':"""This is the exponent of the data value (such as if the data is in thousandths, it would read -3).  This is coded, not a direct value, and only read via a table.  It also does not refer to the Unit exponent, as in cubic inches might have an exponent of 3, but that's a different field (Units)""",
        'UnitSystem':"""Read via a table, this gives the Unit System that the units fit into, such as "SI Linear".""",
        'Units':"""This is both the name of the unit, and the exponent (such as cubic inches would be inch^3), and several might apply to one item.  Several units are constructed by giving the parts they decode into, such as FootPounds being represented by units of inches and Pounds (this table is more than a bit screwy).  I know of two methods to decode this field, and since I can make strong arguments for both algorithms, I have decided to use the method that the Linux kernel uses.  If you wish discussion of this, write to chuckr@chuckr.org.  Forget about help from the USB.org folks, unless you have $4,000.00 for a membership, they'll just ignore you.""",
        'UsagePage':"""Each Usage Page is actually a subtable, with from just a few, to hundreds of subitems.  The Usage Page makes sense on it's own, but the Usage field makes no sense without the previous Usage Table to use to interpret it, so the previous UsagePage is always stored, and displayed (separated by a "/") in the Usage field when that item must be decoded.""",
        'Usage':"""This is the coded Usage type.  In association with the previously set UsageTable and a massive set of tables, this yields the usage type for the current item.  The most recently set UsagePage is always stored, then used and displayed (separated by a "/") when a Usage must be decoded.""",
        'CollectionTypes':"""The exact sort of Collection is decoded (from the data field) via a table, and reads out decoded here.""",
        'IOFQualifiers':"""The Main Class Data fields have many data features, and in this field each of these Features (which only apply to Input, Output, and Feature items) is decoded and presented in this list.""",
        'UnitCodingRules':"""There is a basic problem with the USB HID specification, and the way that the rules for encoding Units/UnitExponenets are detailed.  The USB Organization seems to have a "PAY ME!" attitude towards free software authors ($4,000.00 for a membership, and they won't talk to you without that membership), so I could not get an authoritative answer on this question.  Because I can make equally strong arguments for both of the methods I have found, I just used the method chosen by the Linux kernel.  If you have more problems with this, write to "chuckr@chuckr.org" and I'll discuss it with you.""",
        'Credits':"""Two denizens of the FreeBSD-USB mailing list should definitely get the credit here, for listening to me, and offering advice when I got stuck.  Thanks, fellas."""}


    mainItems={8:'Input',9:'Output',0xA:'Collection',0xB:'Feature',0xC:'End Collection'}

    mainCodes = {\
        0x08:{ 0x1:('Data','Constant'), 0x2:('Array','Variable'), 0x4:('Absolute','Relative'),\
            0x8:('No Wrap','Wrap'), 0x10:('Linear','Non Linear'), 0x20:('Preferred State','No Preferred'),\
            0x40:('No Null position','Null state'), 0x80:('Reserved','Reserved'),\
            0x100:('Bit Field','Buffered Bytes')\
        },0x09:{\
            0x1:('Data','Constant'), 0x2:('Array','Variable'), 0x4:('Absolute','Relative'),\
            0x8:('No Wrap','Wrap'), 0x10:('Linear','Non Linear'), 0x20:('Preferred State','No Preferred'),\
            0x40:('No Null position','Null state'), 0x80:('Non Volatile','Volatile'),\
            0x100:('Bit Field','Buffered Bytes')\
        },0x0B:{\
            0x1:('Data','Constant'), 0x2:('Array','Variable'), 0x4:('Absolute','Relative'),\
            0x8:('No Wrap','Wrap'), 0x10:('Linear','Non Linear'), 0x20:('Preferred State','No Preferred'),\
            0x40:('No Null position','Null state'), 0x80:('Non Volatile','Volatile'),\
            0x100:('Bit Field','Buffered Bytes')\
        },0x0A:{\
            0x0:'Physical (group of axes)', 0x1:'Application (mouse, keyboard)', 0x2:'Logical (interrelated data)',\
            0x3:'Report', 0x4:'Named Array', 0x5:'Usage Switch', 0x6:'Usage Modifier'\
        },0x0C:'End Collection'\
        }

    localItems = {0:'Usage',1:'Usage minimum',2:'Usage maximum',3:'Designator index',4:'Designator minimum',5:'Designator maximum',7:'String index',8:'String minimum',9:'String maximum',0xA:'Delimiter'}

    globalItems = {0:'Usage Page',1:'Logical minimum',2:'Logical maximum',3:'Physical minimum',4:'Physical maximum',5:'Unit exponent',6:'Unit',7:'Report size',8:'Report ID',9:'Report count',0xA:'Push',0xB:'Pop'}

    unitSystem = {1:'SI Linear', 2:'SI Rotation', 3:'English Linear', 4:'English Rotation' }

    unit = { 1:{ 1:'Centimeters', 2:'Grams', 3:'Seconds', 4:'Kelvins', 5:'Amperes', 6:'Candelas'},
            2:{ 1:'Radians', 2:'Grams', 3:'Seconds', 4:'Kelvins', 5:'Amperes', 6:'Candelas'},
            3:{ 1:'Inches', 2:'Slugs', 3:'Seconds', 4:'Fahrenheit', 5:'Amperes', 6:'Candelas'},
            4:{ 1:'Degrees', 2:'Slugs', 3:'Seconds', 4:'Fahrenheit', 5:'Amperes', 6:'Candelas'}}

    exponents = {0x5:'5',0x6:'6',0x7:'7',0x8:'-8',0x9:'-9',0xA:'-7',0xB:'-5',0xC:'-4',0xD:'-3',0xE:'-2',0xF:'-1'}

    # for Generic Desktop Controls (Page 27, 0x01)
    usages1 = {0x1:'Pointer',
            0x2:'Mouse', 0x4:'Joystick', 0x5:'Gamepad', 0x6:'Keyboard', 0x7:'Keypad', 0x8:'Multi-axis Controller',
            0x30:'X', 0x31:'Y', 0x32:'Z', 0x33:'Rx', 0x34:'Ry', 0x35:'Rz', 0x36:'Slider', 0x37:'Dial',
            0x38:'Wheel', 0x39:'Hat switch', 0x3A:'Counted Buffer', 0x3B:'Byte Count', 0x3C:'Motion Wakeup',
            0x3D:'Start', 0x3E:'Select', 0x40:'Vx', 0x41:'Vy', 0x42:'Vz', 0x43:'Vbrx', 0x44:'Vbry',
            0x45:'Vbrz', 0x46:'Vno', 0x47:'Feature Notification', 0x80:'System Control', 0x81:'System Power Down',
            0x82:'System Sleep', 0x83:'System Wake Up', 0x84:'System Context Menu', 0x85:'System Main Menu',
            0x86:'System App Menu', 0x87:'System Menu Help', 0x88:'System Menu Exit', 0x89:'System Menu Select',
            0x8A:'System Menu Right', 0x8B:'System Menu Left', 0x8C:'System Menu Up', 0x8D:'System Menu Down',
            0x8E:'System Cold Restart', 0x8F:'System Warm Restart', 0x90:'D-pad Up', 0x90:'D-pad Down',
            0x90:'D-pad Right', 0x90:'D-pad Left', 0xA0:'System Dock', 0xA1:'System Undock',
            0xA2:'System Setup', 0xA3:'System Break', 0xA4:'System Debugger Break', 0xA5:'Application Break',
            0xA6:'Application Debugger Break', 0xA7:'System Speaker Mute', 0xA8:'System Hibernate',
            0xB0:'System Display Invert', 0xB1:'System Display Internal', 0xB2:'System Display External',
            0xB3:'System Display Both', 0xB4:'System Display Dual', 0xB5:'System Display Toggle Int/Ext',
            0xB6:'System Display Swap Pri/Sec', 0xB7:'System Display LCD Autoscale'}
    
    # for Simulation Controls Page (Page 36, 0x02)
    usages2 = {0x01:'Flight Simulation Device', 0x02:'Automobile Simulation Device', 0x03:'Tank Simulation Device',
        0x04:'Spaceship Simulation Device', 0x05:'Submarine Simulation Device',
        0x06:'Sailing Simulation Device', 0x07:'Motircycle Simulation Device',
        0x08:'Sports Simulation Device', 0x09:'Airplane Simulation Device',
        0x0A:'Helicopter Simulation Device', 0x0B:'Magic Carpet Simulation Device',
        0x0C:'Bicycle Simulation Device', 0x20:'Flight Control Stick', 0x21:'Flight Stick',
        0x22:'Cyclic Control', 0x23:'Cyclic Trim', 0x24:'Flight Yoke', 0x25:'Track Control', 0xB0:'Aileron',
        0xB1:'Aileron Trim', 0xB2:'Anti-Torque Control', 0xB3:'Autopilot Enable', 0xB4:'Chaff Release',
        0xB5:'Collective Control', 0xB6:'Dive Brake', 0xB7:'Electronic Countermeasures',
        0xB8:'Elevator', 0xB9:'Elevator Trim', 0xBA:'Rudder', 0xBB:'Throttle', 0xBC:'Flight Communications',
        0xBD:'Flare Release', 0xBE:'Landing Gear', 0xBF:'Toe Brake', 0xC0:'Trigger', 0xC1:'Weapons Arm',
        0xC2:'Weapons Select', 0xC3:'Wing Flaps', 0xC4:'Accelerator', 0xC5:'Brake', 0xC6:'Clutch',
        0xC7:'Shifter', 0xC8:'Steering', 0xC9:'Turret Direction', 0xCA:'Barrel Elevation',
        0xCB:'Dive Plane', 0xCC:'Ballast', 0xCD:'Bicycle Crank', 0xCE:'Handle Bars', 0xCF:'Front Brake',
        0xD0:'Rear Brake'}

    # VR Controls Page (Page 43, 0x03)
    usages3 = {0x01:'Belt', 0x02:'Body Suit', 0x03:'Flexor', 0x04:'Glove', 0x05:'Head Tracker',
        0x06:'Head Mounted Display', 0x07:'Hand Tracker', 0x08:'Oculometer', 0x09:'Vest',
        0x0A:'Animatronic Device', 0x20:'Stereo Enable', 0x21:'Display Enable'}

    # Sports Controls Page (Page 45, 0x04)
    usages4 = {0x01:'Baseball Bat', 0x02:'Golf Club', 0x03:'Rowing Machine', 0x04:'Treadmill', 0x30:'Oar',
        0x31:'Slope', 0x32:'Rate', 0x33:'Stick Speed', 0x34:'Stick Face Angle', 0x35:'StickHeel/Toe',
        0x36:'Stick Follow Through', 0x37:'Stick Tempo', 0x38:'Stick Type', 0x39:'Stick Height',
        0x50:'Putter', 0x51:'1 Iron', 0x52:'2 Iron', 0x53:'3 Iron', 0x54:'4 Iron', 0x55:'5 Iron',
        0x56:'6 Iron', 0x57:'7 Iron', 0x58:'8 Iron', 0x59:'9 Iron', 0x5A:'10 Iron', 0x5B:'11 Iron',
        0x5C:'Sand Wedge', 0x5D:'Loft Wedge', 0x5E:'Power Wedge', 0x5F:'1 Wood', 0x60:'3 Wood',
        0x61:'5 Wood', 0x62:'7 Wood', 0x63:'9 Wood'}

    # Games Controls Page (Page 48, 0x05)
    usages5 = { 0x01:'3D Game Controller', 0x02:'Pinball Device', 0x03:'Gun Device', 0x20:'Point Of View',
        0x21:'Turn Right/Left', 0x22:'Pitch Forward/Backward', 0x23:'Roll Right/Left', 0x24:'Move Right/Left',
        0x25:'Move Forward/Backward', 0x26:'Move Up/Down', 0x27:'Lean Right/Left', 0x28:'Lean Forward/Backward',
        0x29:'Height of POV', 0x2A:'Flipper', 0x2B:'Secondary Flipper', 0x2C:'Bump', 0x2D:'New Game',
        0x2E:'Shoot Ball', 0x2F:'Player', 0x30:'Gun Bolt', 0x31:'Gun Clip', 0x32:'Gun Selector',
        0x33:'Gun Single Shot', 0x34:'Gun Burst', 0x35:'Gun Automatic', 0x36:'Gun Safety',
        0x37:'Gamepad Fire/Jump', 0x39:'Gamepad Trigger'}

    # Generic Device Controls Page (Page 52, 0x06)
    usages6 = {0x20:'Battery Strength',0x21:'Wireless Channel',0x22:'Wireless ID'}

    # Keyboard/Keypad Page (Page 53, 0x07)
    usages7 = { 0x01:'Kybd ErrorRollOver', 0x02:'Kybd POSTFail', 0x03:'Kybd ErrorUndefined', 0x04:'Kybd a/A',
        0x05:'Kybd b/B', 0x06:'Kybd c/C', 0x07:'Kybd d/D', 0x08:'Kybd e/E', 0x09:'Kybd f/F', 0x0A:'Kybd g/G',
        0x0B:'Kybd h/H', 0x0C:'Kybd i/I', 0x0D:'Kybd j/J', 0x0E:'Kybd k/K', 0x0F:'Kybd l/L', 0x10:'Kybd m/M',
        0x11:'Kybd n/N', 0x12:'Kybd o/O', 0x13:'Kybd p/P', 0x14:'Kybd q/Q', 0x15:'Kybd r/R', 0x16:'Kybd s/S',
        0x17:'Kybd t/T', 0x18:'Kybd u/U', 0x19:'Kybd v/V', 0x1A:'Kybd w/W', 0x1B:'Kybd x/X', 0x1C:'Kybd y/Y',
        0x1D:'Kybd z/Z', 0x1E:'Kybd 1/!', 0x1F:'Kybd 2/@', 0x20:'Kybd 3/#', 0x21:'Kybd 4/$', 0x22:'Kybd 5/%',
        0x23:'Kybd 6/^', 0x24:'Kybd 7/&', 0x25:'Kybd 8/*', 0x26:'Kybd 9/(', 0x27:'Kybd 0/)',
        0x28:'Kybd Return (ENTER)', 0x29:'Kybd ESCAPE', 0x2A:'Kybd DELETE (Backspace)', 0x2B:'Kybd Tab',
        0x2C:'Kybd Spacebar', 0x2D:'Kybd -/_', 0x2E:'Kybd =/+', 0x2F:'Kybd [/{', 0x30:'Kybd ]/}',
        0x31:'Kybd \/|', 0x32:'Kybd Non-US/\~', 0x33:'Kybd ;/:', 0x34:'Kybd \'/\"',
        0x35:'Kybd GraveAccent/Tilde', 0x36:'Kybd \,/<', 0x37:'Kybd \./>', 0x38:'Kybd ForwardSlash/QuestionMark',
        0x39:'Kybd CapsLock', 0x3A:'Kybd F1', 0x3B:'Kybd F3', 0x3C:'Kybd F3', 0x3D:'Kybd F4',
        0x3E:'Kybd F5', 0x3F:'Kybd F6', 0x40:'Kybd F7', 0x41:'Kybd F8', 0x42:'Kybd F9', 0x43:'Kybd F10',
        0x44:'Kybd F11', 0x45:'Kybd F12', 0x46:'Kybd PrintScreen', 0x47:'Kybd ScrollLock', 0x48:'Kybd Pause',
        0x49:'Kybd Insert', 0x4A:'Kybd Home', 0x4B:'Kybd PageUp', 0x4C:'Kybd DeleteForward', 0x4D:'Kybd End',
        0x4E:'Kybd PageDown', 0x4F:'Kybd RightArrow', 0x50:'Kybd LeftArrow', 0x51:'Kybd DownArrow',
        0x52:'Kybd UpArrow', 0x53:'Kybd NumLock/Clear', 0x54:'Keypad /', 0x55:'Keypad *', 0x56:'Keypad -',
        0x57:'Keypad +', 0x58:'Keypad ENTER', 0x59:'Keypad 1/End', 0x5A:'Keypad 2/DownArrow',
        0x5B:'Keypad 3/PageDn', 0x5C:'Keypad 4/LeftArrow', 0x5D:'Keypad 5', 0x5E:'Keypad 6/RightArrow',
        0x5F:'Keypad 7/Home', 0x60:'Keypad 8/UpArrow', 0x61:'Keypad 9/PageUp', 0x62:'Keypad 0/Insert',
        0x63:'Keypad ./Delete', 0x64:'Kybd Non-US/|', 0x65:'Kybd Application', 0x66:'Kybd Power',
        0x67:'Kybd =', 0x68:'Kybd F13', 0x69:'Kybd F14', 0x6A:'Kybd F15', 0x6B:'Kybd F16', 0x6C:'Kybd F17',
        0x6D:'Kybd F18', 0x6E:'Kybd F19', 0x6F:'Kybd F20', 0x70:'Kybd F21', 0x71:'Kybd f@@', 0x72:'Kybd F23',
        0x73:'Kybd F24', 0x74:'Kybd Execute', 0x75:'Kybd Help', 0x76:'Kybd Menu', 0x77:'Kybd Select',
        0x78:'Kybd Stop', 0x79:'Kybd Again', 0x7A:'Kybd Undo', 0x7B:'Kybd Cut', 0x7C:'Kybd Copy',
        0x7D:'Kybd Paste', 0x7E:'Kybd Find', 0x7F:'Kybd Mute', 0x80:'Kybd Volume Up', 0x81:'Kybd Volume Down',
        0x82:'Kybd Locking Capslock', 0x83:'Kybd Locking Numlock', 0x84:'Kybd Locking ScrollLock',
        0x85:'Keypad Comma', 0x86:'Keypad Equal Sign', 0x87:'Kybd Intl-1', 0x88:'Kybd Intl-2',
        0x89:'Kybd Intl-3', 0x8A:'Kybd Intl-4', 0x8B:'Kybd Intl-5', 0x8C:'Kybd Intl-6', 0x8D:'Kybd Intl-7',
        0x8E:'Kybd Intl-8', 0x8F:'Kybd Intl-9', 0x90:'Kybd LANG1', 0x91:'Kybd LANG2', 0x92:'Kybd LANG3',
        0x93:'Kybd LANG4', 0x94:'Kybd LANG5', 0x95:'Kybd LANG6', 0x96:'Kybd LANG7', 0x97:'Kybd LANG8',
        0x98:'Kybd LANG9', 0x99:'Kybd Alternate Erase', 0x9A:'Kybd SysReq/Attention',
        0x9B:'Kybd Cancel', 0x9C:'Kybd Clear', 0x9D:'Kybd Prior', 0x9E:'Kybd Return', 0x9F:'Kybd Separator',
        0xA0:'Kybd Out', 0xA1:'Kybd Oper', 0xA2:'Kybd Clear/Again', 0xA3:'Kybd CrSel/Props',
        0xA4:'Kybd ExSel', 0xB0:'Keypad 00', 0xB1:'Keypad 000', 0xB2:'Kybd Thousands Separator',
        0xB3:'Kybd Decimal Separator', 0xB4:'Kybd Currency Unit', 0xB5:'Kybd Currency Sub-Unit',
        0xB6:'Keypad (', 0xB7:'Keypad )', 0xB8:'Keypad {', 0xB9:'Keypad }', 0xBA:'Keypad Tab',
        0xBB:'Keypad Backspace', 0xBC:'Keypad A', 0xBD:'Keypad B', 0xBE:'Keypad C', 0xBF:'Keypad D',
        0xC0:'Keypad E', 0xC1:'Keypad F', 0xC2:'Keypad XOR', 0xC3:'Keypad ^', 0xC4:'Keypad %',
        0xC5:'Keypad <', 0xC6:'Keypad >', 0xC7:'Keypad &', 0xC8:'Keypad &&', 0xC9:'Keypad |',
        0xCA:'Keypad ||', 0xCB:'Keypad :', 0xCC:'Keypad #', 0xCD:'Keypad Space', 0xCE:'Keypad @',
        0xCF:'Keypad !', 0xD0:'Keypad Memory Store', 0xD1:'Keypad Memory Recall', 0xD2:'Keypad Memory Clear',
        0xD3:'Keypad Memory Add', 0xD4:'Keypad Memory Subtract', 0xD5:'Keypad Memory Multiply',
        0xD6:'Keypad Memory Divide', 0xD7:'Keypad +/-', 0xD8:'Keypad Clear', 0xD9:'Keypad Clear Entry',
        0xDA:'Keypad Binary', 0xDB:'Keypad Octal', 0xDC:'Keypad Decimal', 0xDD:'Keypad Hexadecimal',
        0xE0:'Kybd LeftControl', 0xE1:'Kybd LeftShift', 0xE2:'Kybd LeftAlt', 0xE3:'Kybd LeftGUI',
        0xE4:'Kybd RightControl', 0xE5:'Kybd RightShift', 0xE6:'Kybd RightAlt', 0xE7:'Kybd RightGUI'}

    # LED Usage Page (Page 61, 0x08)
    usages8 = {
        0x01:'NumLock', 0x02:'CapsLock', 0x03:'ScrollLock', 0x04:'Compose', 0x05:'Kana', 0x06:'Power',
        0x07:'Shift', 0x08:'DoNotDisturb', 0x09:'Mute', 0x0A:'ToneEnable', 0x0B:'HighCutFilter',
        0x0C:'LowCutFilter', 0x0D:'EqualizerEnable', 0x0E:'SoundFieldOn', 0x0F:'SurroundOn',
        0x10:'Repeat', 0x11:'Stereo', 0x12:'SamplingRateDetect', 0x13:'Spinning', 0x14:'CAV',
        0x15:'CLV', 0x16:'RecordingFormatDetect', 0x17:'OffHook', 0x18:'Ring', 0x19:'MessageWaiting',
        0x1A:'DataMode', 0x1B:'BatteryOperation', 0x1C:'BatteryOK', 0x1D:'BattteryLow', 0x1E:'Speaker',
        0x1F:'HeadSet', 0x20:'Hold', 0x21:'Microphone', 0x22:'Coverage', 0x23:'NightMode', 0x24:'SendCalls',
        0x25:'CallPickup', 0x26:'Conference', 0x27:'StandBy', 0x28:'CameraOn', 0x29:'CameraOff', 0x2A:'OnLine',
        0x2B:'OffLine', 0x2C:'Busy', 0x2D:'Ready', 0x2E:'PaperOut', 0x2F:'PaperJam', 0x30:'Remote',
        0x31:'Forward', 0x32:'Reverse', 0x33:'Stop', 0x34:'Rewind', 0x35:'FastForward', 0x36:'Play',
        0x37:'Pause', 0x38:'Record', 0x39:'Error', 0x3A:'UsageSelectedIndicator',
        0x3B:'UsageInUseIndicator', 0x3C:'UsageMultiModeIndicator', 0x3D:'IndicatorOn', 0x3E:'IndicatorFlash',
        0x3F:'IndicatorSlowBlink', 0x40:'IndicatorFastBlink', 0x41:'IndicatorOff', 0x42:'FlashOnTime',
        0x43:'SlowBlinkOnTime', 0x44:'SlowBlinkOffTime', 0x45:'FastBlinkOnTime', 0x46:'FastBlinkOffTime',
        0x47:'UsageIndicatorColor', 0x48:'IndicatorRed', 0x49:'IndicatorGreen', 0x4A:'IndicatorAmber',
        0x4B:'GenericIndicator', 0x4C:'SystemSuspend', 0x4D:'ExternalPowerConnected'}

    # both Button Page and Ordinal page are total mappings (0 to FFFF),  so handle
    # them via functions

    usages9 = {}
    usagesA = {}

    # Telephony Usage Page (Page 69, 0x0B)
    usagesB = { 0x01:'Phone', 0x02:'Answering Machine', 0x03:'Message Controls', 0x04:'Handset', 0x05:'Headset',
        0x06:'Telephony Key Pad', 0x07:'Programmable Button', 0x20:'Hook Switch', 0x21:'Flash',
        0x22:'Feature', 0x23:'Hold', 0x24:'Redial', 0x25:'Transfer', 0x26:'Drop', 0x27:'Park',
        0x28:'Forward Calls', 0x29:'Alternate Function', 0x2A:'Line', 0x2B:'Speaker Phone', 0x2C:'Conference',
        0x2D:'Ring Enable', 0x2E:'Ring Select', 0x2F:'Phone Mute', 0x30:'Caller ID', 0x31:'Send',
        0x50:'Speed Dial', 0x51:'Store Number', 0x52:'Recall Number', 0x53:'Phone Directory', 0x70:'Voice Mail',
        0x71:'Screen Calls', 0x72:'Do Not Disturb', 0x73:'Message', 0x74:'Answer On/Off',
        0x90:'Inside Dial Tone', 0x91:'Outside Dial Tone', 0x92:'Inside Ring Tone', 0x93:'Outside Ring Tone',
        0x94:'Priority Ring Tone', 0x95:'Inside Ringback', 0x96:'Priority Ringback', 0x97:'Line Busy Tone',
        0x98:'Reorder Tone', 0x99:'Call Waiting Tone', 0x9A:'Confirmation Tone 1', 0x9B:'Confirmation Tone 2',
        0x9C:'Tones Off', 0x9D:'Outside Ringback', 0x9E:'Ringer', 0xB0:'Phone Key 0', 0xB1:'Phone Key 1',
        0xB2:'Phone Key 2', 0xB3:'Phone Key 3', 0xB4:'Phone Key 4', 0xB5:'Phone Key 5', 0xB6:'Phone Key 6',
        0xB7:'Phone Key 7', 0xB8:'Phone Key 8', 0xB9:'Phone Key 9', 0xBA:'Phone Key Star',
        0xBB:'Phone Key Pound', 0xBC:'Phone Key A', 0xBD:'Phone Key B', 0xBE:'Phone Key C', 0xBF:'Phone Key D', }

    # Consumer Usage Page (Page 75, 0x0C)
    usagesC = { 0x001:'Consumer Control', 0x002:'Numeric Key Pad', 0x003:'Programmable Buttons', 0x004:'Microphone',
        0x005:'Headphone', 0x006:'Graphic Equalizer', 0x020:'+10', 0x021:'+100', 0x022:'AM/PM', 0x030:'Power',
        0x031:'Reset', 0x032:'Sleep', 0x033:'Sleep After', 0x034:'Sleep Mode', 0x035:'Illumination',
        0x036:'Function Buttons', 0x040:'Menu', 0x041:'Menu Pick', 0x042:'Menu Up', 0x043:'Menu Down',
        0x044:'Menu Left', 0x045:'Menu Right', 0x046:'Menu Escape', 0x047:'Menu Value Increase',
        0x048:'Menu Value Decrease', 0x060:'Data On Screen', 0x061:'Closed Caption',
        0x062:'Closed Caption Select', 0x063:'VCR/TV', 0x064:'Broadcast Mode', 0x065:'Snapshot', 0x066:'Still',
        0x080:'Selection', 0x081:'Assign Selection', 0x082:'Mode Step', 0x083:'Recall Last',
        0x084:'Enter Channel', 0x085:'Order Movie', 0x086:'Channel', 0x087:'Media Selection',
        0x088:'Media Select Computer', 0x089:'Media Select TV', 0x08A:'Media Select WWW',
        0x08B:'Media Select DVD', 0x08C:'Media Select Telephone', 0x08D:'Media Select Program Guide',
        0x08E:'Media Select Video Phone', 0x08F:'Media Select Gammes', 0x090:'Media Select Messages',
        0x091:'Media Select CD', 0x092:'Media Select VCR', 0x093:'Media Select Tuner', 0x094:'Quit',
        0x095:'Help', 0x096:'Media Select Tape', 0x097:'Media Select Cable', 0x098:'Media Select Satellite',
        0x099:'Media Select Security', 0x09A:'Media Select Home', 0x09B:'Media Select Call',
        0x09C:'Channel Increment', 0x09D:'Channel Decrement', 0x09E:'Media Select SAP', 0x0A0:'VCR Plus',
        0x0A1:'Once', 0x0A2:'Daily', 0x0A3:'Weekly', 0x0A4:'Monthly', 0x0B0:'Play', 0x0B1:'Pause',
        0x0B2:'Record', 0x0B3:'Fast Forward', 0x0B4:'Rewind', 0x0B5:'Scan Next Track',
        0x0B6:'Scan Previous Track', 0x0B7:'Stop', 0x0B8:'Eject', 0x0B9:'Random Play', 0x0BA:'Select Disc',
        0x0BB:'Enter Disc', 0x0BC:'Repeat', 0x0BD:'Tracking', 0x0BE:'Track Normal', 0x0BF:'Slow Tracking',
        0x0C0:'Frame Forward', 0x0C1:'Frame Back', 0x0C2:'Mark', 0x0C3:'Clear Mark', 0x0C4:'Repeat From Mark',
        0x0C5:'Return To Mark', 0x0C6:'Search Mark Forward', 0x0C7:'Search Mark Backwards',
        0x0C8:'Counter Reset', 0x0C9:'Show Counter', 0x0CA:'Tracking Increment', 0x0CB:'Tracking Decrement',
        0x0CC:'Stop/Eject', 0x0CD:'Play/Pause', 0x0CE:'Play/Skip', 0x0E0:'lume', 0x0E1:'Balance',
        0x0E2:'Mute', 0x0E3:'Bass', 0x0E4:'Treble', 0x0E5:'Bass Boost', 0x0E6:'Surround Mode', 0x0E7:'Loudness',
        0x0E8:'MPX', 0x0E9:'Volume Increment', 0x0EA:'Volume Decrement', 0x0F0:'Speed Select',
        0x0F1:'Playback Speed', 0x0F2:'Standard Play', 0x0F3:'Long Play', 0x0F4:'Extended Play', 0x0F5:'Slow',
        0x100:'Fan Enable', 0x101:'Fan Speed', 0x102:'Light Enable', 0x103:'Light Illumination Level',
        0x104:'Climate Control Enable', 0x105:'Room Temperature', 0x106:'Security Enable', 0x107:'Fire Alarm',
        0x108:'Police Alarm', 0x109:'Proximity', 0x10A:'Motion', 0x10B:'Duress Alarm', 0x10C:'Holdup Alarm',
        0x10D:'Medical Alarm', 0x150:'Balance Right', 0x151:'Balance Left', 0x152:'Bass Increment',
        0x153:'Bass Decrement', 0x154:'Treble Increment', 0x155:'Treble Decrement', 0x160:'Speaker System',
        0x161:'Channel Left', 0x162:'Channel Right', 0x163:'Channel Center', 0x164:'Channel Front',
        0x165:'Channel Center Front', 0x166:'Channel Side', 0x167:'Channel Surround',
        0x168:'Channel Low Frequency Enhancement', 0x169:'Channel Top', 0x16A:'Channel Uknnown',
        0x170:'Sub-channel', 0x171:'Sub-Channel Increment', 0x172:'Sub-Channel Decrement',
        0x173:'Alternate Audio Increment', 0x174:'Alternate Audio Decrement',
        0x180:'Application Launch Buttons', 0x181:'AL Launch Button Configuration Tool',
        0x182:'AL Programmable Button Configuration', 0x183:'AL Consumer Control Configuration',
        0x184:'AL Word Processor', 0x185:'AL Text Editor', 0x186:'AL Spreadsheet', 0x187:'AL Graphics Editor',
        0x188:'AL Presentation App', 0x189:'AL Database App', 0x18A:'AL Email Reader', 0x18B:'AL Newsreader',
        0x18C:'AL Voicemail', 0x18D:'AL Contacts/Address Book', 0x18E:'AL Calendar/Schedule',
        0x18F:'AL Task/Project Manager', 0x190:'AL Log/Journal/Timecard', 0x191:'AL Checkbook/Finnance',
        0x192:'AL Calculator', 0x193:'AL A/V Capture/Playback', 0x194:'AL Local Machine Browser',
        0x195:'AL LAN/WAN Browser', 0x196:'AL Internet Browser', 0x197:'AL Remote Networking/ISP Connect',
        0x198:'AL Network Conference', 0x199:'AL Network Chat', 0x19A:'AL Telephony/Dialer', 0x19B:'AL Logon',
        0x19C:'AL Logoff', 0x19D:'AL Logon/Logoff', 0x19E:'AL Terminal Lock/Screensaver',
        0x19F:'AL Control Panel', 0x1A0:'AL Command Line Processor/Run', 0x1A1:'AL Process/Task Manager',
        0x1A2:'AL Select Task/Application', 0x1A3:'AL Next Task/Application',
        0x1A4:'AL Previous Task/Application', 0x1A5:'AL Preemptive Halt Task/Application',
        0x1A6:'AL Integrated Help Center', 0x1A7:'AL Documents', 0x1A8:'AL Thesaurus', 0x1A9:'AL Dictionary',
        0x1AA:'AL Desktop', 0x1AB:'AL Spell Check', 0x1AC:'AL Grammar Check', 0x1AD:'AL Wireless Status',
        0x1AE:'AL Kayboard Layout', 0x1AF:'AL Virus Protection', 0x1B0:'AL Encryption', 0x1B1:'AL Screen Saver',
        0x1B2:'AL Alarms', 0x1B3:'AL Clock', 0x1B4:'AL File Browser', 0x1B5:'AL File Status',
        0x200:'Generic GUI Application Control', 0x201:'AC New', 0x202:'AC Open', 0x203:'AC Close',
        0x204:'AC Exit', 0x205:'AC Maximize', 0x206:'AC Minimize', 0x207:'AC Save', 0x208:'AC Print',
        0x209:'AC Properties', 0x21A:'AC Undo', 0x21B:'AC Copy', 0x21C:'AC Cut', 0x21D:'AC Paste',
        0x21E:'AC Select All', 0x21F:'AC Find', 0x220:'AC Find & Replace', 0x221:'AC Search', 0x222:'AC Go To',
        0x223:'AC Home', 0x224:'AC Back', 0x225:'AC Forward', 0x226:'AC Stop', 0x227:'AC Refresh',
        0x228:'AC Previous Link', 0x229:'AC Next Link', 0x22A:'AC Bookmarks', 0x22B:'AC History',
        0x22C:'AC Subscriptions', 0x22D:'AC Zoom In', 0x22E:'AC Zoom Out', 0x22F:'AC Zoom',
        0x230:'AC Full Screen View', 0x231:'AC Normal View', 0x232:'AC View Toggle', 0x233:'AC Scroll Up',
        0x234:'AC Scroll Down', 0x235:'AC Scroll', 0x236:'AC Pan Left', 0x237:'AC Pan Right', 0x238:'AC Pan',
        0x239:'AC New Window', 0x23A:'AC Tile Horizontally', 0x23B:'AC Tile Vertically', 0x23C:'AC Format',
        0x23D:'AC Edit', 0x23E:'AC Bold', 0x23F:'AC Italics', 0x240:'AC Underline', 0x241:'AC Strikethrough',
        0x242:'AC Subscript', 0x243:'AC Superscript', 0x244:'AC All Caps', 0x245:'AC Rotate',
        0x246:'AC Resize', 0x247:'AC Flip Horizontal', 0x248:'AC Flip Vertical', 0x249:'AC Mirror Horizontal',
        0x24A:'AC Mirror Vertical', 0x24B:'AC Font Select', 0x24C:'AC Font Color', 0x24D:'AC Font Size',
        0x24E:'AC Left', 0x24F:'AC Justify Center H', 0x250:'AC Justify Right', 0x251:'AC Justify Block H',
        0x252:'AC Justify Top', 0x253:'AC Justify Center V', 0x254:'AC Justify Bottom',
        0x255:'AC Justify Block V', 0x256:'AC Indent Decrease', 0x257:'AC Indent Increase',
        0x258:'AC Numbered List', 0x259:'AC Restart Numbering', 0x25A:'AC Bulleted List', 0x25B:'AC Promote',
        0x25C:'AC Demote', 0x25D:'AC Yes', 0x25E:'AC No', 0x25F:'AC Cancel', 0x260:'AC Catalog',
        0x261:'AC Buy/Checkout', 0x262:'AC Add To Cart', 0x263:'AC Expand', 0x264:'AC Expand All',
        0x265:'AC Collapse', 0x266:'AC Collapse All', 0x267:'AC Print Preview', 0x268:'AC Paste Special',
        0x269:'AC Insert Mode', 0x26A:'AC Delete', 0x26B:'AC Lock', 0x26C:'AC Unlock', 0x26D:'AC Protect',
        0x26E:'AC Unprotect', 0x26F:'AC Attach Comment', 0x270:'AC Delete Comment', 0x271:'AC View Comment',
        0x272:'AC Select Word', 0x273:'AC Select Sentence', 0x274:'AC Select Paragraph',
        0x275:'AC Select Column', 0x276:'AC Select Row', 0x277:'AC Select Table', 0x278:'AC Select Object',
        0x279:'AC Redo/Repeat', 0x27A:'AC Sort', 0x27B:'AC Sort Ascending', 0x27C:'AC Sort Descending',
        0x27D:'AC Filter', 0x27E:'AC Set Clock', 0x27F:'AC View Clock', 0x280:'AC Select Time Zone',
        0x281:'AC Edit Time Zones', 0x282:'AC Set Alarm', 0x283:'AC Clear Alarm', 0x284:'AC Snooze Alarm',
        0x285:'AC Reset Alarm', 0x286:'AC Synchronize', 0x287:'AC Send/Receive', 0x289:'AC Send / To',
        0x28A:'AC Reply', 0x28B:'AC Reply All', 0x28C:'AC Forward Msg', 0x28D:'AC Attach File',
        0x28E:'AC Upload', 0x28F:'AC Download (Save Target As)', 0x290:'AC Set Borders',
        0x291:'AC Insert Row', 0x292:'AC Insert Column', 0x293:'AC Insert File', 0x294:'AC Insert Picture',
        0x295:'AC Insert Object', 0x296:'AC Insert Symbol', 0x297:'AC Save and Close', 0x298:'AC Rename',
        0x299:'AC Merge', 0x29A:'AC Split', 0x29B:'AC Distribute Horizontally',
        0x29C:'AC Distribute Vertically', }

    # Digitizer Page (Page 102, 0x0D)
    usagesD = { 0x01:'Digitizer', 0x02:'Pen', 0x03:'Light Pen', 0x04:'Touch Screen', 0x05:'Touch Pad',
        0x06:'White Board', 0x07:'Coordinate Measuring Machine', 0x08:'3D Digitizer', 0x09:'Stereo Plotter',
        0x0A:'Articulated Arm', 0x0B:'Armature', 0x0C:'Multiple Point Digitizer', 0x0D:'Free Space Wand',
        0x20:'Stylus', 0x21:'Puck', 0x22:'Finger', 0x30:'Tip Pressure', 0x31:'Barrel Pressure',
        0x32:'In Range', 0x33:'Touch', 0x34:'Untouch', 0x35:'Tap', 0x36:'Quality', 0x37:'Data Valid',
        0x38:'Transducer Index', 0x39:'Tablet Function Keys', 0x3A:'Program Change Keys',
        0x3B:'Battery Strength', 0x3C:'Invert', 0x3D:'X Tilt', 0x3E:'Y Tilt', 0x3F:'Azimuth',
        0x40:'Altitude', 0x41:'Twist', 0x42:'Tip Switch', 0x43:'Secondary Tip Switch',
        0x44:'Barrel Switch', 0x45:'Eraser', 0x46:'Tablet Pick', }

    # Alphanumeric Display Usage Page (Page 108, 0x14)
    usagesE = {}

    # ASCII Display Character Set (Page 110, 0x??)
    usagesF = {}

    # Example 7-Segment Font Values For Hex Digits (Page 118, 0x??)
    #usages6 = {}

    # Medical Instruments, page 108, 0x14
    usages14 = { 0x01:'Alphanumeric Display', 0x20:'Display Attributes Report', 0x21:'ASCII Character Set',
        0x22:'Data Read Back', 0x23:'Font Read Back', 0x24:'Display Control Report', 0x25:'Clear Display',
        0x26:'Display Enable', 0x27:'Screen Saver Delay', 0x28:'Screen Saver Enable', 0x29:'Vertical Scroll',
        0x2A:'Horizontal', 0x2B:'Character Report', 0x2C:'Display Data', 0x2D:'Display Status',
        0x2E:'Stat Not Ready', 0x2F:'Stat Ready', 0x30:'Err Not a Loadable Character',
        0x31:'Err Font Cannot Be Read', 0x32:'Cursor Position Report', 0x33:'Row', 0x34:'Column',
        0x35:'Rows', 0x36:'Columns', 0x37:'Cursor Pixel Positioning', 0x38:'Cursor Mode', 0x39:'Cursor Enable',
        0x3A:'Cursor Blink', 0x3B:'Font Report', 0x3C:'Font Data', 0x3D:'Character Width',
        0x3E:'Character Height', 0x3F:'Character Spacing Horizontal', 0x40:'Character Spacing Vertical',
        0x41:'Unicode Character Set', 0x42:'Font 7-Segment', 0x43:'7-Segment Direct Map',
        0x44:'Font 14-Segment', 0x45:'14-Segment Direct Map', 0x46:'Display Brightness', 0x47:'Display Contrast',
        0x48:'Character Attribute', 0x49:'Attribute Readback', 0x4A:'Attribute Data',
        0x4B:'Char Attr Enhance', 0x4C:'Char Attr Underline', 0x4D:'Char Attr Blink' }

    # Medical Instrument Page 0x40 (Page 120, 0x40)
    usages40 = { 0x01:'Medical UltraSound', 0x20:'VCR/Acquisition', 0x21:'Freeze/Thaw', 0x22:'Clip Store',
        0x23:'Update', 0x24:'Next', 0x25:'Save', 0x26:'Print', 0x27:'Microphone Enable', 0x40:'Cline',
        0x41:'Transmit Power', 0x42:'Volume', 0x43:'Focus', 0x44:'Depth', 0x60:'Soft Step Primary',
        0x61:'Soft Step Secondary', 0x70:'Depth Gain Compensation', 0x80:'Zoom Select', 0x81:'Zoom Adjust',
        0x82:'Spectral Doppler Mode Select', 0x83:'Spectral Adjust', 0x84:'Color Doppler Mode Select',
        0x85:'Color Doppler Adjust', 0x86:'Motion Mode Select', 0x87:'Motion Mode Adjust', 0x88:'2-D Mode Select',
        0x89:'2-D Mode Adjust', 0xA0:'Soft Control Select', 0xA1:'Soft Control Adjust' }

    # Monitor Pages 0x80 (from FreeBSD's /usr/share/misc/usb_hid_usages file)  128
    usages80 = { 0x01:'Monitor Control', 0x02:'EDID Information', 0x03:'VDIF Information', 0x04:'VESA Version'}

    # Monitor Enumerated Values 0x81 (from FreeBSD's /usr/share/misc/usb_hid_usages file) 129
    # usages81 = { 0x%X:'ENUM %d'}

    # VESA Virtual Controls 0x82 (from FreeBSD's /usr/share/misc/usb_hid_usages file) 130
    usages82 = { 0x01:'Degauss', 0x10:'Brightness', 0x12:'Contrast', 0x16:'Red Video Gain', 0x18:'Green Video Gain',
        0x1A:'Blue Video Gain', 0x1C:'Focus', 0x20:'Horizontal Position', 0x22:'Horizontal Size',
        0x24:'Horizontal Pincushion', 0x26:'Horizontal Pincushion Balance', 0x28:'Horizontal Misconvergence',
        0x2A:'Horizontal Linearity'}

    # Power Device 0x83 (from FreeBSD's /usr/share/misc/usb_hid_usages file) 132
    usages84 = { 0x01:'Degauss', 0x10:'Brightness', 0x12:'Contrast', 0x16:'Red Video Gain', 0x18:'Green Video Gain',
        0x1A:'Blue Video Gain', 0x1C:'Focus', 0x20:'Horizontal Position', 0x22:'Horizontal Size',
        0x24:'Horizontal Pincushion', 0x26:'Horizontal Pincushion Balance', 0x28:'Horizontal Misconvergence',
        0x2A:'Horizontal Linearity', 0x2C:'Horizontal Linearity Balance', 0x30:'Vertical Position',
        0x32:'Vertical Size', 0x34:'Vertical Pincushion', 0x36:'Vertical Pincushion Balance',
        0x38:'Vertical Misconvergence', 0x3A:'Vertical Linearity', 0x3C:'Vertical Linearity Balance',
        0x40:'Parallelogram Distortion', 0x42:'Trapezoidal Distortion', 0x44:'Tilt',
        0x46:'Top Corner Distortion Control', 0x48:'Top Corner Distortion Balance',
        0x4A:'Bottom Corner Distortion Control', 0x4C:'Bottom Corner Distortion Balance',
        0x56:'Horizontal Moir<E9>', 0x58:'Vertical Moir<E9>', 0x5E:'Input Level Select',
        0x60:'Input Source Select', 0x6C:'Red Video Black Level', 0x6E:'Green Video Black Level',
        0x70:'Blue Video Black Level', 0xA2:'Auto Size Center', 0xA4:'Polarity Horizontal Synchronization',
        0xA6:'Polarity Vertical Synchronization', 0xA8:'Synchronization Type', 0xAA:'Screen Orientation',
        0xAC:'Horizontal Frequency', 0xAE:'Vertical Frequency', 0xB0:'Settings',
        0xCA:'On Screen Display', 0xD4:'Stereo Mode'}

    # Battery System Page 0x85  (from FreeBSD's /usr/share/misc/usb_hid_usages file) 133
    usages85 = { 0x00:'Undefined', 0x01:'SMBBatteryMode', 0x02:'SMBBatteryStatus', 0x03:'SMBAlarmWarning',
        0x04:'SMBChargerMode', 0x05:'SMBChargerStatus', 0x06:'SMBChargerSpecInfo', 0x07:'SMBSelectorState',
        0x08:'SMBSelectorPresets', 0x09:'SMBSelectorInfo', 0x10:'OptionalMfgFunction1', 0x11:'OptionalMfgFunction2',
        0x12:'OptionalMfgFunction3', 0x13:'OptionalMfgFunction4', 0x14:'OptionalMfgFunction5',
        0x15:'ConnectionToSMBus', 0x16:'OutputConnection', 0x17:'ChargerConnection', 0x18:'BatteryInsertion',
        0x19:'Usenext', 0x1A:'OKToUse', 0x1B:'BatterySupported', 0x1C:'SelectorRevision',
        0x1D:'ChargingIndicator', 0x28:'ManufacturerAccess', 0x29:'RemainingCapacityLimit',
        0x2A:'RemainingTimeLimit', 0x2B:'AtRate', 0x2C:'CapacityMode', 0x2D:'BroadcastToCharger',
        0x2E:'PrimaryBattery', 0x2F:'ChargeController', 0x40:'TerminateCharge', 0x41:'TermminateDischarge',
        0x42:'BelowRemainingCapacityLimit', 0x43:'RemainingTimeLimitExpired', 0x44:'Charging',
        0x45:'Discharging', 0x46:'FullyCharged', 0x47:'FullyDischarged', 0x48:'ConditioningFlag',
        0x49:'AtRateOK', 0x4A:'SMBErrorCode', 0x4B:'NeedReplacement', 0x60:'AtRateTimeToFull',
        0x61:'AtRateTimeToEmpty', 0x62:'AverageCurrent', 0x63:'Maxerror', 0x64:'RelativeStateOfCharge',
        0x65:'AbsoluteStateOfCharge', 0x66:'RemainingCapacity', 0x67:'FullChargeCapacity',
        0x68:'RunTimeToEmpty', 0x69:'AverageTimeToEmpty', 0x6A:'AverageTimeToFull', 0x6B:'CycleCount',
        0x80:'BattPackModelLevel', 0x81:'InternalChargeController', 0x82:'PrimaryBatterySupport',
        0x83:'DesignCapacity', 0x84:'SpecificationInfo', 0x85:'ManufacturerDate', 0x86:'SerialNumber',
        0x87:'ManufacturerName', 0x88:'Devicename', 0x89:'DeviceChemistery', 0x8A:'ManufacturerData',
        0x8B:'Rechargeable', 0x8C:'WarningCapacityLimit', 0x8D:'CapacityGranularity1',
        0x8E:'CapacityGranularity2', 0x8F:'OEMInformation', 0xC0:'InhibitCharge', 0xC1:'EnablePolling',
        0xC2:'ResetToZero', 0xD0:'ACPresent', 0xD1:'BatteryPresent', 0xD2:'PowerFail', 0xD3:'AlarmInhibited',
        0xD4:'ThermistorUnderRange', 0xD5:'ThermistorHot', 0xD6:'ThermistorCold', 0xD7:'ThermistorOverRange',
        0xD8:'VoltageOutOfRange', 0xD9:'CurrentOutOfRange', 0xDA:'CurrentNotRegulated',
        0xDB:'VoltageNotRegulated', 0xDC:'MasterMode', 0xF0:'ChargerSelectorSupport',
        0xF1:'ChargerSpec', 0xF2:'Level2', 0xF3:'Level3'}

    # Bar Code Scanner Page 0x8C (from FreeBSD's /usr/share/misc/usb_hid_usages file) 140
    usages8C = { 0x01:'Bar Code Badge Reader', 0x02:'Bar Code Scanner', 0x03:'Dumb Bar Code Scanner',
        0x04:'Cordless Scanner Base', 0x05:'Bar Code Scanner Cradle', 0x10:'Attribute Report',
        0x11:'Settings Report', 0x12:'Scanned Data Report', 0x13:'Raw Scanned Data Report',
        0x14:'Trigger Report', 0x15:'Status Report', 0x16:'UPC/EAN Control Report',
        0x17:'EAN 2/3 Label Control Report', 0x18:'Code 39 Control Report',
        0x19:'Interleaved 2 of 5 Control Report', 0x1A:'Standard 2 of 5 Control Report',
        0x1B:'MSI Plessey Control Report', 0x1C:'Codabar Control Report', 0x1D:'Code 128 Control Report',
        0x1E:'Misc 1D Control Report', 0x1F:'2D Control Report', 0x30:'Aiming/Pointer Mode',
        0x31:'Bar Code Present Sensor', 0x32:'Class 1A Laser', 0x33:'Class 2 Laser', 0x34:'Heater Present',
        0x35:'Contact Scanner', 0x36:'Electronic Article Surveillance Notification',
        0x37:'Constant Electronic Article', 0x38:'Error Indication', 0x39:'Fixed Beeper',
        0x3A:'Good Decode Indication', 0x3B:'Hands Free Scanning', 0x3C:'Intrinsically Safe',
        0x3D:'Klasse Eins Laser', 0x3E:'Long Range Scanner', 0x3F:'Mirror Speed Control',
        0x40:'Not On File Indication', 0x41:'Programmable Beeper', 0x42:'Triggerless', 0x43:'Wand',
        0x44:'Water Resistant', 0x45:'Multi-Range Scanner', 0x46:'Proximity Sensor', 0x4D:'Fragment Decoding',
        0x4E:'Scanner Read Confidence', 0x4F:'Data Prefix', 0x50:'Prefix AIMI', 0x51:'Prefix None',
        0x52:'Prefix Proprietary', 0x55:'Active Time', 0x56:'Aiming Laser Pattern', 0x57:'Bar Code Present',
        0x58:'Beeper State', 0x59:'Laser On Time', 0x5A:'Laser State', 0x5B:'Lockout Timeout',
        0x5C:'Motor State', 0x5D:'Motor Timeout', 0x5E:'Power On Reset Scanner', 0x5F:'Prevent Read of Barcodes',
        0x60:'Initiate Barcode Read', 0x61:'Trigger State', 0x62:'Trigger Mode',
        0x63:'Trigger Mode Blinking Laser On', 0x64:'Trigger Mode Continuous Laser On',
        0x65:'Trigger Mode Laser on while Pulled', 0x66:'TriggerModeLaser stays on after Trigger release',
        0x6D:'Commit Parameters to NVM', 0x6E:'Parameter Scanning', 0x6F:'Parameters Changed',
        0x70:'Set parapeter default values', 0x75:'Scanner In Cradle', 0x76:'Scanner In Range',
        0x7A:'Aim Duration', 0x7B:'Good Read Lamp Duration', 0x7C:'Good Read Lamp Intensity',
        0x7D:'Good Read LED', 0x7E:'Good Read Tone Frequence', 0x7F:'Good Read Tone Length',
        0x80:'Good Read Tone Volume', 0x82:'No Read Message', 0x83:'Not On File Volume'}

    # Weighing Devices Page 0x8D (from FreeBSD's /usr/share/misc/usb_hid_usages file) 141
    usages8D = { 0x01:'Weighing Device', 0x20:'Scale Device', 0x21:'Scale Class I Metric',
        0x22:'Scale Class I Metric', 0x23:'Scale Class II Metric', 0x24:'Scale Class III Metric',
        0x25:'Scale Class IIIL Metric', 0x26:'Scale Class IV Metric', 0x27:'Scale Class III English',
        0x28:'Scale Class IIIL English', 0x29:'Scale Class IV English', 0x2A:'Scale Class Generic',
        0x30:'Scale Attribute Report', 0x31:'Scale Control Report', 0x32:'Scale Data Report',
        0x33:'Scale Status Report', 0x34:'Scale Weight Limit Report', 0x35:'Scale Statistics Report',
        0x40:'Data Weight', 0x41:'Data Scaling', 0x50:'Weight Unit', 0x51:'Weight Unit Milligram',
        0x52:'Weight Unit Gram', 0x53:'Weight Unit Kilogram', 0x54:'Weight Unit Carats', 0x55:'Weight Unit Taels',
        0x56:'Weight Unit Grains', 0x57:'Weight Unit Pennyweights', 0x58:'Weight Unit Metric Ton',
        0x59:'Weight Unit Avoir Ton', 0x5A:'Weight Unit Troy Ounce', 0x5B:'Weight Unit Ounce',
        0x5C:'Weight Unit Pound', 0x60:'Calibration Count', 0x61:'Re-Zero Count', 0x70:'Scale Status',
        0x71:'Scale Status Fault', 0x72:'Scale Status Stable at Center of Zero', 0x73:'Scale Status In Motion',
        0x74:'Scale Status Weight Stable', 0x75:'Scale Status Under Zero', 0x76:'Scale Status Over Weight Limit',
        0x77:'Scale Status Requires Rezeroing', 0x80:'Zero Scale', 0x81:'Enforced Zero Return'}

    # Magnetic Stripe Reader 0x8E (from FreeBSD's /usr/share/misc/usb_hid_usages file) 142
    usages8E = { 0x01:'MSR Device Read-Only', 0x11:'Track 1 Length', 0x12:'Track 2 Length',
        0x13:'Track 3 Length', 0x14:'Track JIS Length', 0x20:'Track Data', 0x21:'Track 1 Data',
        0x22:'Track 2 Data', 0x23:'Track 3 Data', 0x24:'Track JIS Data'}

    # Camera Control Pages 0x8F (from FreeBSD's /usr/share/misc/usb_hid_usages file) 143
    usages8F = {}

    # Arcade Device Pages 0x90 (from FreeBSD's /usr/share/misc/usb_hid_usages file) 144
    usages90 = {}

    usages = {0x01:('Generic Desktop Controls',usages1),
            0x02:('Simulation Controls',usages2),
        0x03:('VR Controls',usages3),
        0x04:('Sport Controls',usages4),
        0x05:('Game Controls',usages5),
        0x06:('Generic Device Controls',usages6),
        0x07:('Keyboard/keypad',usages7),
        0x08:('LED Page',usages8),
        0x09:('Button Page',usages9),
        0x0A:('Ordinal',usagesA),
        0x0B:('Telephony Device',usagesB),
        0x0C:('Consumer',usagesC),
        0x0D:('Digitizers',usagesD),
        0x0E:('Reserved',usagesE),
        0x0F:('PID Page',usagesF),
        0x10:('UNICODE, direct mapped',usages2),
        0x14:('Alphanumeric Display',usages14),
        0x40:('Medical Instrument',usages40),
        0x80:('Monitor Page',usages80),
        0x82:('Monitor Page',usages82),
        0x84:('Power Page',usages84),
        0x85:('Power Page',usages85),
        0x8C:('Bar Code Scanner',usages8C),
        0x8D:('Scale Page',usages8D),
        0x8E:('Mag Strip Reading Device',usages8E),
        0x8F:('Point of Sale Pages',usages8F),
        0x90:('Arcade Device',usages90)}



if __name__ == '__main__':

    print tables.usages1[2]
